# encoding: utf-8

'''🛠️ JPL LabCAS Utilities: constants.'''


DEFAULT_SOLR_URL    = 'https://localhost:8984/solr/'
PROTOCOL_RDF_SOURCE = 'https://edrn.jpl.nasa.gov/cancerdataexpo/rdf-data/protocols/@@rdf'
DMCC_TOKEN_API      = 'https://www.compass.fhcrc.org/edrnauthapi/auth/token'
DMCC_IMAGE_EVENTS   = 'https://www.compass.fhcrc.org/edrnrestws/Data/GetAllImageEvents'
DEFAULT_PROTOCOL    = 430  # 430 is for Prostate MRI
DEFAULT_TOKEN_ID    = '59502649ee224c2fa58362d336e4c92c'